<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 12, 2020
 */

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Api_Public_Resources extends Model_Api
{
    const API_ENDPOINT_RESOURCES = '/api/resource_management/v4/resources';
    const API_ENDPOINT_RESOURCE_STATUSES = '/api/resource_management/v4/resource_statuses';
    const QUERY_AGENT_ATTRIBUTES = '?include_attribute_namespaces=agent';

    // res = response
    const RES_ATTRIBUTES = 'attributes';
    const RES_KVS = 'kvs';
    const RES_KEY = 'key';
    const RES_VALUE = 'value';
    const RES_CONTEXT = 'context';
    const RES_TYPE = 'type';
    const RES_NAME = 'name';

    const TYPE_RESOURCE_MACHINE = 'resource.machine';

    const AGENT_ATTRIBUTE_KEY_COMPONENTS = 'components';
    const AGENT_ATTRIBUTE_KEY_FEATURES = 'features';
    const AGENT_ATTRIBUTE_KEY_OS_FAMILY = 'os_family';
    const AGENT_ATTRIBUTE_KEY_OS_NAME = 'os_name';
    const AGENT_ATTRIBUTE_KEY_OS_ARCH = 'os_arch';
    const AGENT_ATTRIBUTE_KEY_OS_CAPS = 'os_caps';
    const AGENT_ATTRIBUTE_KEY_OS_SERVICE_PACK = 'os_service_pack';
    const AGENT_ATTRIBUTE_KEY_OS_PRODUCT_TYPE = 'os_product_type';
    const AGENT_ATTRIBUTE_KEY_OS_SUITE_MASK = 'os_suite_mask';
    const AGENT_ATTRIBUTE_KEY_SKU = 'sku';
    const AGENT_ATTRIBUTE_KEY_OS_VERSION_MAJOR = 'os_version_major';
    const AGENT_ATTRIBUTE_KEY_OS_VERSION_MINOR = 'os_version_minor';
    const AGENT_ATTRIBUTE_KEY_MAC_ADDRESSES = 'mac_addresses';
    const AGENT_ATTRIBUTE_KEY_MEMORY_SIZE = 'memory_size';
    const AGENT_ATTRIBUTE_KEY_PROCESSOR_FREQUENCY = 'processor_frequency';
    const AGENT_ATTRIBUTE_KEY_PROCESSOR_NAME = 'processor_name';
    const AGENT_ATTRIBUTE_KEY_CHASSIS = 'chassis';
    const AGENT_ATTRIBUTE_VM_HOST_TYPE = 'vm_host_type';
    const AGENT_ATTRIBUTE_KEY_ESX_ADDRESS = 'esx_address';
    const AGENT_ATTRIBUTE_RESIDENTAL_ADDRESSES = 'residental_addresses';
    const AGENT_ATTRIBUTE_KEY_INSIDE_VIRTUAL = 'inside_virtual';
    const AGENT_ATTRIBUTE_KEY_TZ_OFFSET = 'tz_offset';
    const AGENT_ATTRIBUTE_VERSION = 'version';
    const AGENT_ATTRIBUTE_ENABLED = 'enabled';
    const AGENT_ATTRIBUTE_IS_ONLINE = 'is_online';

    const AGENT_ATTRIBUTE_KEYS = array(
        self::AGENT_ATTRIBUTE_KEY_COMPONENTS,
        self::AGENT_ATTRIBUTE_KEY_FEATURES,
        self::AGENT_ATTRIBUTE_KEY_OS_FAMILY,
        self::AGENT_ATTRIBUTE_KEY_OS_NAME,
        self::AGENT_ATTRIBUTE_KEY_OS_ARCH,
        self::AGENT_ATTRIBUTE_KEY_OS_CAPS,
        self::AGENT_ATTRIBUTE_KEY_OS_SERVICE_PACK,
        self::AGENT_ATTRIBUTE_KEY_OS_PRODUCT_TYPE,
        self::AGENT_ATTRIBUTE_KEY_OS_SUITE_MASK,
        self::AGENT_ATTRIBUTE_KEY_SKU,
        self::AGENT_ATTRIBUTE_KEY_OS_VERSION_MAJOR,
        self::AGENT_ATTRIBUTE_KEY_OS_VERSION_MINOR,
        self::AGENT_ATTRIBUTE_KEY_MAC_ADDRESSES,
        self::AGENT_ATTRIBUTE_KEY_MEMORY_SIZE,
        self::AGENT_ATTRIBUTE_KEY_PROCESSOR_FREQUENCY,
        self::AGENT_ATTRIBUTE_KEY_PROCESSOR_NAME,
        self::AGENT_ATTRIBUTE_KEY_CHASSIS,
        self::AGENT_ATTRIBUTE_VM_HOST_TYPE,
        self::AGENT_ATTRIBUTE_KEY_ESX_ADDRESS,
        self::AGENT_ATTRIBUTE_RESIDENTAL_ADDRESSES,
        self::AGENT_ATTRIBUTE_KEY_INSIDE_VIRTUAL,
        self::AGENT_ATTRIBUTE_KEY_TZ_OFFSET,
        self::AGENT_ATTRIBUTE_VERSION,
        self::AGENT_ATTRIBUTE_ENABLED,
        self::AGENT_ATTRIBUTE_IS_ONLINE,
    );

    /**
     * get resource data from api with matching hostname.
     *
     * @return Model_Resources_ResourceMachine
     */
    public function getResourceByHostname(): Model_Resources_ResourceMachine
    {
        $query = http_build_query(array(
            'include_attributes' => true, // include attributes
            'limit' => 500, // limit response to 500 (Nexcess gateway limit)
            'type' => 'resource.machine'
        ));

        $apiEndpoint = self::API_ENDPOINT_RESOURCE_STATUSES . "?$query";
        $response = $this->_getOAuthClient()->sendGetRequest($apiEndpoint);
        $items = $response['items'];

        foreach ($items as $item) {
            $context = $item[self::RES_CONTEXT];
            $api_resource_hostname = $context[self::RES_NAME];

            if ($this->_isHostnameMatch($api_resource_hostname)) {
                return new Model_Resources_ResourceMachine($context);
            }
        }

        throw new Exception_Acronis_ResourceNotFound('API does not include machine with hostname: ' . gethostname());
    }

    /**
     * @param string $api_resource_hostname
     */
    private function _isHostnameMatch($api_resource_hostname)
    {
        return $api_resource_hostname === gethostname();
    }

    /**
     * @return string
     */
    public function getAgentMemorySizeInBytes(): string
    {
        return $this->_getAgentAttribute(self::AGENT_ATTRIBUTE_KEY_MEMORY_SIZE);
    }

    /**
     * @param string $attribute_key
     * @return mixed
     */
    private function _getAgentAttribute($attribute_key)
    {
        if (!in_array($attribute_key, self::AGENT_ATTRIBUTE_KEYS)) {
            throw new IWorx_Exception_DoingItWrong("attribute key invalid: $attribute_key");
        }

        $instance_id = $this->_getInstanceId();
        $endpoint = self::API_ENDPOINT_RESOURCES . "/$instance_id" . self::QUERY_AGENT_ATTRIBUTES;
        $res = $this->_getOAuthClient()->sendGetRequest($endpoint);
        $attributes = $res[self::RES_ATTRIBUTES];

        // grabbing first item because it only includes one - because of query in request
        $agent_attributes = $attributes[0][self::RES_KVS];

        foreach ($agent_attributes as $attribute) {
            if ($attribute[self::RES_KEY] === $attribute_key) {
                return $attribute[self::RES_VALUE];
            }
        }

        throw new Exception('Api endpoint does not seem to have agent memory size! ' . $endpoint);
    }
}
