# Acronis Plugin
## Software Configuration
1. The following ports must be open: 443, 8443, 44445, 5060 and 7770-7800. The following link can be used to verify that all necessary ports are open: [Acronis Backup Cloud: Connection Verification Tool](https://kb.acronis.com/content/47678)

## Installing the plugin
Install the plugin by running the following command:
`yum install interworx-plugin-acronis`

The plugin will automatically be updated to its latest version whenever interworx is updated.

## Activating the Plugin
1. Log into NodeWorx and navigate to NodeWorx > Plugins.
2. Acronis should be listed under Backup / Restore. Select Edit.
3. Enter the acronis credentials & change the status to "Enabled".
4. Select save & refresh the page.
5. Navigate to Server > Acronis and select "Enable Backup". The backup agent will be installed and backup will be enabled. If the backup agent has been installed manually, selecting "Enable Backup" will make sure that all configuration files are located in the right directory.

## CLI
#### Interworx CLI Utility
The CLI utility provides the same functionality as the web interface: http://docs.interworx.com/interworx/cli/index.php.

#### Other Commands
1. Install the backup agent:
```
~iworx/bin/php /home/interworx/plugins/acronis/bin/acronisinstaller.php --install
```
2. Uninstall the backup agent:
```
~iworx/bin/php /home/interworx/plugins/acronis/bin/acronisinstaller.php --uninstall
```

## Using the Plugin with Existing Backup Agents & Plans

### Installing the Plugin
There is no difference when it comes to installing the plugin. Follow the instructions above.
The installation will automatically recognize if an agent exists on the machine and skip this step.

### Using an Existing Backup Plan
If the agent already has a backup plan enabled, the plugin will recognize this plan and not attempt to
replace it. If no plan is assigned to the resource, you can select between existing plans via the CLI.
Setting backup to "on" in the UI will look for existing plans with pre/post scripts enabled
(necessary for database restoration) or create a new one if none is found.


### Selecting Existing Plan in CLI
1. Display enabled plan:
```
[root@test ~]# nodeworx -u -c acronis -a getBackupPlanDetails -n
{
  "enabled":true,
  "id":"ADA8EBEE-E904-4AE0-8537-746927136BBC",
  "name":"Interworx Default",
  "time":{
    "minute":0,
    "hour":2,
    "second":0
  },
  "days":[
    "sunday",
    "monday",
    "tuesday",
    "wednesday",
    "thursday",
    "friday",
    "saturday"
  ]
}
```
2. List compatible plans:
```
[root@test ~]# nodeworx -u -c acronis -a listCompatiblePlans -n
[
  {
    "name":"New protection plan",
    "ID":"CAA674F9-32E5-4005-B274-B72E96962968"
  }
]
```

3. Activate specific plan:
```
[root@acronis-v4-iw6 ~]# nodeworx -u -c acronis -a activatePlan --plan_id=CAA674F9-32E5-4005-B274-B72E96962968 -n
{
  "enabled":true,
  "id":"CAA674F9-32E5-4005-B274-B72E96962968",
  "name":"New protection plan",
  "time":{
    "minute":15,
    "hour":16,
    "second":0
  },
  "days":[
    "monday",
    "tuesday",
    "wednesday",
    "thursday",
    "friday"
  ],
  "message":"Backup plan successfully activated"
}
```

### Enabling Database Restoration on Custom Plans
Database restoration is enabled by default if using InterWorx default backup plan.
If using a custom plan, pre/post scripts must be enabled for database restoration to work.

1. Log into the acronis cloud (may be accessed via plugin by selecting "Go to backup console" in the dashboard tab).
2. Select your device, then "Protect"
3. Select to edit your plan
4. Select "Backup options"
5. Select "Pre-post commands"
6. Set "Execute a command before the backup" to "Yes"
    * Command or batch file path on the machine with an agent: `/usr/lib/Acronis/BackupAndRecovery/webcpprecapture`
    * Arguments: `{RESOURCE_ID}`
7. Set "Execute a command after the backup" to "Yes"
    * Command or batch file path on the machine with an agent: `/usr/lib/Acronis/BackupAndRecovery/webcppostcapture`
    * Arguments: `{RESOURCE_ID}`
8. Select "Done"
9. Select "Save"




## Plugin Functionality
### NodeWorx
1. Single-signon to Acronis Backup Cloud.
2. Run backup.
3. Turn backup on/off.
4. View last backup status & time.
5. View when the next backup is scheduled.
6. View logs associated with acronis backups.
7. Traverse & restore files/folders.
8. Restore databases.

### SiteWorx
1. Traverse & restore files/folders.
2. Restore databases.


## Troubleshooting
1. If links are unresponsive or page content doesn't load, update the file permissions by running the following CLI command:
~iworx/bin/cvspermsfix.pex
2. If encountering issues when activating the plugin, make sure that the data center URL is correct. Make sure that the URL does not contain any trailing /. E.g. `https://cloud.acronis.com` will work, while `https://cloud.acronis.com/` may result in an error. Also make sure to include the protocol, `https://`.
3. The required packages should be installed automatically when enabling the plugin, but if the installation fails we recommend verifying that they got intsalled manually:
    1. Packages for building kernel modules: `kernel-devel elfutils-devel elfutils-libelf-devel`. Make sure that the package version matches the kernel version:
    ```
    Yum list installed | grep kernel-devel
    cat /proc/version
    ```

    If versions do not match, update the kernel and reboot.
    ```
    Yum update kernel
    Reboot
    ```

    2. GNU Compiler Collection (`GCC`) compiler system. `GCC` version must be the same as the version used to compile the kernel:
    ```
    Gcc -v
    cat /proc/version
    ```
    3. `Make` tool
    ```
    make -v
    ```
    For more information & troubleshooting steps: https://kb.acronis.com/content/55052

