<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Aug 23, 2021
 */
require_once(__DIR__ . '/AcronisTestCase.php');
/**
 * @package InterWorx
 * @subpackage
 */
abstract class Acronis_NodeworxTest extends AcronisTestCase
{

    const PARENT_DIR_PATH_TEST = '/home/acronis_recovery_test';
    const DIR_CONTENT_EXPECTED_COUNT = 400;
    const PLAN_NO_PRESCRIPT = 'NOT DEFAULT'; // MAKE SURE THIS EXISTS IN CLOUD
    const ACRONISTESTUSER = 'acronistest';

    /**
     * Throw exception if plugin not activated.
     * TODO fail test instead?
     */
    protected function _validatePluginActivated()
    {
        $AcronisPlugin = IW::PluginManager()->getPlugin('acronis');
        if (!$AcronisPlugin->isEnabled()) {
            $this->fail('PLUGIN NOT ENABLED! ENABLE ACRONIS BEFORE RUNNING TESTS');
        }
    }

    /**
     * @param string $action
     * @param array  $args
     * @return Reply
     */
    protected function _sendCtrlRequest($action, array $args = array())
    {
        return $this->_sendNwCtrlRequest($action, $args);
    }

    /**
     * await plan activation
     */
    protected function _awaitActivation()
    {
        $attempts = 0;
        $maxAttempts = 10;

        while (!$this->_isActivated() && $attempts < $maxAttempts) {
            sleep(10);
            $attempts++;
        }

        if (!$this->_isActivated()) {
            throw new Exception('PLAN NOT AcTIVATED');
        }
    }

    /**
     * Delete acronis_temp dir
     */
    protected function _deleteTempDirIfExists()
    {
        if (Model_AcronisFileSys::isDir(Model_Recovery_Files::NW_DOWNLOAD_DIR_PATH)) {
            Model_AcronisFileSys::rm(Model_Recovery_Files::NW_DOWNLOAD_DIR_PATH);
        }
    }

    /**
     * @return array
     */
    protected function _getUnmappedDbNames()
    {
        $Backup = $this->_getBackup();
        $dbs = $Backup->getUnmappedDatabases();

        $dbNames = array_map(function ($db) {
            return $db['name'];
        }, $dbs);

        return $dbNames;
    }
}
