import { updateBreadcrumbs, removeBreadcrumbs } from './breadcrumbs.js';
import { listBackups, loadDirContent, loadRootContent, listDatabases } from '../ctrlActions.js';
import { listRecoveryPointOptions } from './recoveryPoint.js';

export const router = {
    activate: (controller) => activateRouter(controller),
    route: (newRoute) => reroute(newRoute)
}

const reroute = (newRoute) => {
    window.location.hash = newRoute;
}

const activateRouter = (controller) => {
    removeHashAndRegiterDefaultEventOnLoad(controller);
    loadRouteOnHashChange(controller);
}

const removeHashAndRegiterDefaultEventOnLoad = (controller) => {
    $(window).off('load');
    $(window).on('load', function () {
        reroute('');
        listBackups(controller);
    });
}


const loadRouteOnHashChange = (controller) => {
    $(window).off('hashchange');
    $(window).on('hashchange', () => {
        let route = getRouteFromUri();
        let routes = getRoutes(controller);
        let functionToCall = routes[route];

        functionToCall();
        handleBreadcrumbs(routes, functionToCall);
    });
}

const handleBreadcrumbs = (object, value) => {
    let key = getKeyByValue(object, value);

    if (key === '/') {
        removeBreadcrumbs();
    } else {
        let params = getParams();
        updateBreadcrumbs(params.breadcrumbs);
    }
}

const getKeyByValue = (object, value) => Object.keys(object).find(key => object[key] === value);

const getRoutes = (controller) => {
    return {
        '/': () => listBackups(controller),
        '/:id': listRecoveryPointOptions,
        '/:id/files': () => loadRootContent(controller),
        '/:id/files/:id': () => loadDirContent(controller),
        '/:id/databases': () => listDatabases(controller)
    };
};

const getRouteFromUri = () => {
    let params = getParams();

    let route = '/';

    if (params.backupName) {
        route += ':id';
    }

    if (params.type) {
        route += `/${params.type}`;
    }

    if (params.dirPath) {
        route += '/:id';
    }

    return route;
}


const getHashParams = () => decodeURI(window.location.hash.substr(1));

const getParams = () => {
    let breadcrumbPath = getHashParams();
    let splitPath = breadcrumbPath.split('/');

    let backupName = splitPath[0];
    let type = splitPath[1];
    let dirPath = breadcrumbPath.split('/').slice(2).join('/');

    return {
        breadcrumbs: breadcrumbPath,
        dirPath: dirPath,
        type: type,
        backupName: backupName
    };
}

const getSlashIndexes = () => {
    let indexes = [];
    let hashParams = getHashParams();
    if (hashParams) {
        indexes.push(0);

        //get further params.
        for (let i = 0; i < hashParams.length; i++) {
            let currentLetter = hashParams[i];
            if (currentLetter === '/') {
                indexes.push(i);
            }
        }
    }



    return indexes;
}

export const getRecoveryType = () => getParams().type;

export const getBreadcrumbPath = () => getParams().breadcrumbs;

export const getDirectoryPath = () => getParams().dirPath;